# Copyright © 2022 Arrikto Inc.  All Rights Reserved.

"""Kale SDK.

This script uses an ML pipeline to train and serve an SKLearn Model.
"""

from kale.ml import Signature
from kale.sdk import pipeline, step
from kale.common import mlmdutils, artifacts

from sklearn.feature_extraction import text
from sklearn.naive_bayes import MultinomialNB
from sklearn.datasets import fetch_20newsgroups
from sklearn.model_selection import train_test_split
from sklearn.feature_extraction.text import TfidfVectorizer


@step(name="data_loading")
def load_split_dataset():
    """Fetch 20newgroup dataset."""
    # load the data
    newsgroups_dataset = fetch_20newsgroups(random_state=42)
    x = newsgroups_dataset.data
    y = newsgroups_dataset.target

    # split the dataset
    x_train, x_test, y_train, y_test = train_test_split(x, y, test_size=.2,
                                                        random_state=42)
    return x_train, y_train, x_test, y_test


@step(name="data_preprocess")
def preprocess(x_train, x_test):
    """Preprocess the input data."""
    # get stopwords
    stop_words = text.ENGLISH_STOP_WORDS
    # TF-IDF vectors
    vectorizer = TfidfVectorizer(stop_words=stop_words)
    vectors_train = vectorizer.fit_transform(x_train)
    vectors_test = vectorizer.transform(x_test)
    return vectors_train, vectors_test


@step(name="model_training")
def train(x, y):
    """Train a MultinomialNB model."""
    classifier = MultinomialNB(alpha=.01)
    model = classifier.fit(x, y)
    return model


@step(name="register_model")
def register_model(model, x, y):
    mlmd = mlmdutils.get_mlmd_instance()

    signature = Signature(
        input_size=[1] + list(x[0].shape),
        output_size=[1] + list(y[0].shape),
        input_dtype=x.dtype,
        output_dtype=y.dtype)

    model_artifact = artifacts.SklearnModel(
        model=model,
        description="A simple MultinomialNB model",
        version="1.0.0",
        author="Kale",
        signature=signature,
        tags={"app": "sklearn-model"}).submit_artifact()

    mlmd.link_artifact_as_output(model_artifact.id)
    return model_artifact.id


@pipeline(name="classification", experiment="sklearn-model")
def ml_pipeline():
    """Run the ML pipeline."""
    x_train, y_train, x_test, y_test = load_split_dataset()
    vectors_train, vectors_test = preprocess(x_train, x_test)
    model = train(vectors_train, y_train)
    register_model(model, vectors_train, y_train)


if __name__ == "__main__":
    ml_pipeline()
